/* Tooltips */
table.rtxcalendar .day {
    position: relative;
    z-index: 1;
    padding: 3px 3px 3px 6px;
    margin-top: 4px;
    margin-bottom: 4px;
    width:120%;
    height: 1.75rem;
    z-index: 4;
}

table.rtxcalendar .day.last-day {
    width: 100%;
}

table.rtxcalendar .day:hover {
    z-index: 5;
    color:#000;
}
table.rtxcalendar .day span.tip {
    display: none;
    text-align: left;
}
table.rtxcalendar div.day div.event-info:hover span.tip{
    display: block;
    position: absolute;
    top:1rem; left:24px; width:350px;
    border: 1px solid #555;
    background-color: #fff;
    padding: 4px;
    font-size: 0.8em;
    color: #505050;
}

table.rtxcalendar td.weekday-7 div.day div.event-info:hover span.tip,
table.rtxcalendar td.weekday-6 div.day div.event-info:hover span.tip,
table.rtxcalendar td.weekday-5 div.day div.event-info:hover span.tip {
    left: unset;
    right: 24px;
}

/* For the full calendar */
table.rtxcalendar {
    width:100%;
    border-collapse: collapse;
    border: 1px solid #d0d0d0;
    margin-bottom: 6px;
}

table.rtxcalendar td {
    border: 1px solid #d7d7d7;
    background: #fff;
    vertical-align: top;
    width: 14%;
}

table.rtxcalendar th {
    border: 1px solid #d7d7d7;
    background: #eef;
}

table.rtxcalendar td .inside-day {
    min-height: 5em;
}

table.rtxcalendar td.offmonth {
    background: #f8f8f8;
    color: #aaa;
}

table.rtxcalendar td.today {
    background: #ffe;
    border: 1px solid #fc6;
}

table.rtxcalendar td.today .calendardate {
    background-color: #fc6;
    width:100%;
}

table.rtxcalendar td.yesterday {
    border-right: none;
}

table.rtxcalendar td.aweekago {
    border-bottom: none;
}

.calendar-sidebar {
    padding-right: 0.5rem;
}

.calendar-content {
    width: auto;
}

.calendar-sidebar .tipimg {
    display: table-cell;
    width: 25px;
}
.calendar-sidebar .tiplegend {
    display: table-cell;
}
.calendar-sidebar .tip {
    display: table-row;
}

a.calendar-toggle-sidebar,
a.calendar-toggle-sidebar.off {
    background: none !important;
    width: 0px !important;
    position: relative;
    float: right;
    top: 300px;
    margin-right: 10px;
}
a.calendar-toggle-sidebar::before {
    content: '';
    display: block;
    width: 8px;
    height: 8px;
    border: solid #B0B3BC;
    border-width: 0 2px 2px 0;
    -webkit-transform: rotate(135deg);
    -ms-transform: rotate(135deg);
    transform: rotate(135deg);
    transition: transform .25s;
    position: absolute;
    top: 1.2em;
}
a.calendar-toggle-sidebar.sidebar-off::before {
    transform: rotate(315deg);
}
.calendar-sidebar-toggle-content {
    float: left;
    width: 230px;
    margin-top: 20px;
}
.calendar-sidebar-toggle-content.sidebar-off {
    width:unset;
}
.calendar-content.sidebar-off {
    margin-left: 20px;
}
.filteronstatus {
    width: 100% !important;
}
.calendar-filter-status-box {
    margin-top: 0px;
}
.calendar-sidebar {
    margin-right: 10px;
}

.calendar-container {
    display: flow-root;
}

.event-icon {
    float: left;
    margin-right: 5px;
}

.event-info a.event-title {
    overflow: hidden;
    max-width: 100%;
    left: 0px;
    padding-left: 21px;
    white-space: nowrap;
    position: absolute;
    color: white;
}

table.rtxcalendar .day.first-day {
    border-radius: 5px 0 0 5px;
}
table.rtxcalendar .day.last-day {
    border-radius: 0 5px 5px 0;
}
table.rtxcalendar .day.first-day.last-day {
    border-radius: 5px;
}

.darkmode table.rtxcalendar * {
    background: unset !important;
}

.darkmode table.rtxcalendar div.day div.event-info:hover span.tip{
    border: 1px solid #555;
    background-color: #2C3539 !important;
    color: #fff !important;
}

.darkmode table.rtxcalendar div.day a {
    color: #fff !important;
}
